# $Id: latex.bzl,v 1.17 2018/08/27 07:13:22 aehlig Exp $

load("//aehlig_rules/view:pdf.bzl", "view_pdf")

LatexInfo = provider()

def _pdf_latex_impl(ctx):
    inputs = depset(transitive=[depset(ctx.files.srcs), depset(ctx.files.main)])
    if LatexInfo in ctx.attr.main:
        inputs = depset(transitive=[inputs,
                                    ctx.attr.main[LatexInfo].referenced])
    for i in ctx.attr.srcs:
        if LatexInfo in i:
            inputs = depset(transitive=[inputs, i[LatexInfo].referenced])
    inputfiles = [f.path + "|" + f.short_path for f in inputs.to_list()]
    if ctx.attr.flavor:
        inputfiles = [ctx.files.flavor[0].path + "|flavor.tex"] + inputfiles
        inputs = depset(transitive=[inputs, depset(ctx.files.flavor)])
        if LatexInfo in ctx.attr.flavor:
            inputs = depset(transitive=[inputs,
                                        ctx.attr.flavor[LatexInfo].referenced])
    output = ctx.actions.declare_file(ctx.label.name + ".pdf")
    inputs_file = ctx.actions.declare_file(ctx.label.name + ".allinputs")
    # Note: we assume that if main is a filegroup, the actual entry
    # point is the first element
    args = [output.path] + [f.short_path for f in ctx.files.main[:1]] \
           + [inputs_file.path]
    ctx.actions.write(inputs_file, "\n".join(inputfiles))
    ctx.actions.run(
        inputs = depset([inputs_file],
                        transitive=[inputs, depset(ctx.files._runpdflatex)]),
        outputs = [output],
        executable = ctx.files._runpdflatex[0].path,
        arguments = args,
        use_default_shell_env = True,
        mnemonic = "PdfLatex",
        progress_message = "Typesetting %s as pdf" % ctx.label,
    )

pdf_latex = rule(
    implementation = _pdf_latex_impl,
    attrs = {
        "main" : attr.label(allow_files=True),
        "srcs" : attr.label_list(allow_files=True),
        "flavor" : attr.label(allow_files=True),
        "_runpdflatex":
        attr.label(cfg="host", allow_files=True,
                   default = Label("//aehlig_rules/latex:runpdflatex")),
    },
    outputs = {"pdf" : "%{name}.pdf"},
)

def latex(name="", main="", srcs=[], flavor=None, visibility=None):
    pdf_latex(name=name, main=main, srcs=srcs, flavor=flavor,
              visibility=visibility)
    view_pdf(name=name)

## Includefiles

def _includefile_impl(ctx):
    output = ctx.actions.declare_file(ctx.label.name + ".tex")
    deps = depset(ctx.files.srcs)
    includes = ["\\input{%s}\n" % f.short_path for f in deps.to_list()]
    ctx.actions.write(output = output, content = "".join(includes))
    return [LatexInfo(referenced = depset([output], transitive=[deps]))]

includefile = rule(
    implementation = _includefile_impl,
    attrs = {
        "srcs" : attr.label_list(allow_files=True),
    },
    outputs = {"tex" : "%{name}.tex"},
)


def latex_include(name="", srcs=[]):
    includefile(name=name, srcs=srcs)

## verbatim files

def latex_verbatim(name="", src=""):
    native.genrule(
        name = name,
        outs = [ name + ".tex" ],
        srcs = [ src ],
        cmd = "echo '\\begin{verbatim}' > $@"
              + " && cat $< >> $@"
              + " && echo '\\end{verbatim}' >> $@",
    )
